@@ -25,6 +25,7 @@
 #include "types.h"
 
 class BufferSyncer : public SyncableObject {
+  SYNCABLE_OBJECT
   Q_OBJECT
 
 public:
@@ -39,34 +40,25 @@ public slots:
   QVariantList initLastSeenMsg() const;
   void initSetLastSeenMsg(const QVariantList &);
 
-  virtual inline void requestSetLastSeenMsg(BufferId buffer, const MsgId &msgId) { emit setLastSeenMsgRequested(buffer, msgId); }
+  virtual inline void requestSetLastSeenMsg(BufferId buffer, const MsgId &msgId) { REQUEST(ARG(buffer), ARG(msgId)) }
 
-  virtual inline void requestRemoveBuffer(BufferId buffer) { emit removeBufferRequested(buffer); }
+  virtual inline void requestRemoveBuffer(BufferId buffer) { REQUEST(ARG(buffer)) }
   virtual void removeBuffer(BufferId buffer);
 
-  virtual inline void requestRenameBuffer(BufferId buffer, QString newName) { emit renameBufferRequested(buffer, newName); }
-  virtual inline void renameBuffer(BufferId buffer, QString newName) { emit bufferRenamed(buffer, newName); }
+  virtual inline void requestRenameBuffer(BufferId buffer, QString newName) { REQUEST(ARG(buffer), ARG(newName)) }
+  virtual inline void renameBuffer(BufferId buffer, QString newName) { SYNC(ARG(buffer), ARG(newName)) emit bufferRenamed(buffer, newName); }
 
-  virtual inline void requestMergeBuffersPermanently(BufferId buffer1, BufferId buffer2) { emit mergeBuffersPermanentlyRequested(buffer1, buffer2); }
+  virtual inline void requestMergeBuffersPermanently(BufferId buffer1, BufferId buffer2) { emit REQUEST(ARG(buffer1), ARG(buffer2)) }
   virtual void mergeBuffersPermanently(BufferId buffer1, BufferId buffer2);
 
-  virtual inline void requestPurgeBufferIds() { emit purgeBufferIdsRequested(); }
+  virtual inline void requestPurgeBufferIds() { REQUEST(NO_ARG); }
 
 signals:
   void lastSeenMsgSet(BufferId buffer, const MsgId &msgId);
-  void setLastSeenMsgRequested(BufferId buffer, const MsgId &msgId);
-
-  void removeBufferRequested(BufferId buffer);
   void bufferRemoved(BufferId buffer);
-
-  void renameBufferRequested(BufferId buffer, QString newName);
   void bufferRenamed(BufferId buffer, QString newName);
-
-  void mergeBuffersPermanentlyRequested(BufferId buffer1, BufferId buffer2);
   void buffersPermanentlyMerged(BufferId buffer1, BufferId buffer2);
 
-  void purgeBufferIdsRequested();
-
 protected slots:
   bool setLastSeenMsg(BufferId buffer, const MsgId &msgId);
   QList<BufferId> bufferIds() const { return _lastSeenMsg.keys(); }
