@@ -34,21 +34,25 @@ public:
   SqliteStorage(QObject *parent = 0);
   virtual ~SqliteStorage();
 
+  virtual AbstractSqlMigrationReader *createMigrationReader();
+
 public slots:
   /* General */
 
   bool isAvailable() const;
   QString displayName() const;
+  virtual inline QStringList setupKeys() const { return QStringList(); }
+  virtual inline QVariantMap setupDefaults() const { return QVariantMap(); }
   QString description() const;
 
   // TODO: Add functions for configuring the backlog handling, i.e. defining auto-cleanup settings etc
 
   /* User handling */
-
   virtual UserId addUser(const QString &user, const QString &password);
-  virtual void updateUser(UserId user, const QString &password);
+  virtual bool updateUser(UserId user, const QString &password);
   virtual void renameUser(UserId user, const QString &newName);
   virtual UserId validateUser(const QString &user, const QString &password);
+  virtual UserId getUserId(const QString &username);
   virtual UserId internalUser();
   virtual void delUser(UserId user);
   virtual void setUserSetting(UserId userId, const QString &settingName, const QVariant &data);
@@ -78,7 +82,7 @@ public slots:
   virtual void setAwayMessage(UserId user, NetworkId networkId, const QString &awayMsg);
   virtual QString userModes(UserId user, NetworkId networkId);
   virtual void setUserModes(UserId user, NetworkId networkId, const QString &userModes);
-  
+
   /* Buffer handling */
   virtual BufferInfo bufferInfo(UserId user, const NetworkId &networkId, BufferInfo::Type type, const QString &buffer = "", bool create = true);
   virtual BufferInfo getBufferInfo(UserId user, const BufferId &bufferId);
@@ -91,12 +95,13 @@ public slots:
   virtual QHash<BufferId, MsgId> bufferLastSeenMsgIds(UserId user);
 
   /* Message handling */
-
-  virtual MsgId logMessage(Message msg);
+  virtual bool logMessage(Message &msg);
+  virtual bool logMessages(MessageList &msgs);
   virtual QList<Message> requestMsgs(UserId user, BufferId bufferId, MsgId first = -1, MsgId last = -1, int limit = -1);
   virtual QList<Message> requestAllMsgs(UserId user, MsgId first = -1, MsgId last = -1, int limit = -1);
 
 protected:
+  inline virtual void setConnectionProperties(const QVariantMap & /* properties */) {}
   inline virtual QString driverName() { return "QSQLITE"; }
   inline virtual QString databaseName() { return backlogFile(); }
   virtual int installedSchemaVersion();
@@ -106,12 +111,51 @@ protected:
 
 private:
   static QString backlogFile();
-  bool isValidNetwork(UserId user, const NetworkId &networkId);
-  bool isValidBuffer(const UserId &user, const BufferId &bufferId);
-  NetworkId getNetworkId(UserId user, const QString &network);
-  void createBuffer(UserId user, const NetworkId &networkId, BufferInfo::Type type, const QString &buffer);
+  void bindNetworkInfo(QSqlQuery &query, const NetworkInfo &info);
+  void bindServerInfo(QSqlQuery &query, const Network::Server &server);
 
+  inline void lockForRead() { _dbLock.lockForRead(); }
+  inline void lockForWrite() { _dbLock.lockForWrite(); }
+  inline void unlock() { _dbLock.unlock(); }
+  QReadWriteLock _dbLock;
   static int _maxRetryCount;
 };
 
+// ========================================
+//  SqliteMigration
+// ========================================
+class SqliteMigrationReader : public SqliteStorage, public AbstractSqlMigrationReader {
+  Q_OBJECT
+
+public:
+  SqliteMigrationReader();
+
+  virtual bool readMo(QuasselUserMO &user);
+  virtual bool readMo(SenderMO &sender);
+  virtual bool readMo(IdentityMO &identity);
+  virtual bool readMo(IdentityNickMO &identityNick);
+  virtual bool readMo(NetworkMO &network);
+  virtual bool readMo(BufferMO &buffer);
+  virtual bool readMo(BacklogMO &backlog);
+  virtual bool readMo(IrcServerMO &ircserver);
+  virtual bool readMo(UserSettingMO &userSetting);
+
+  virtual bool prepareQuery(MigrationObject mo);
+
+  inline int stepSize() { return 50000; }
+
+protected:
+  virtual inline bool transaction() { return logDb().transaction(); }
+  virtual inline void rollback() { logDb().rollback(); }
+  virtual inline bool commit() { return logDb().commit(); }
+
+private:
+  void setMaxId(MigrationObject mo);
+  int _maxId;
+};
+
+inline AbstractSqlMigrationReader *SqliteStorage::createMigrationReader() {
+  return new SqliteMigrationReader();
+}
+
 #endif
