@@ -58,6 +58,21 @@ QVariant ChatItem::data(int role) const {
   return model()->data(index, role);
 }
 
+qint16 ChatItem::posToCursor(const QPointF &pos) const {
+  if(pos.y() > height()) return data(MessageModel::DisplayRole).toString().length();
+  if(pos.y() < 0) return 0;
+
+  QTextLayout layout;
+  initLayout(&layout);
+  for(int l = layout.lineCount() - 1; l >= 0; l--) {
+    QTextLine line = layout.lineAt(l);
+    if(pos.y() >= line.y()) {
+      return line.xToCursor(pos.x(), QTextLine::CursorOnCharacter);
+    }
+  }
+  return 0;
+}
+
 void ChatItem::initLayoutHelper(QTextLayout *layout, QTextOption::WrapMode wrapMode, Qt::Alignment alignment) const {
   Q_ASSERT(layout);
 
@@ -69,10 +84,14 @@ void ChatItem::initLayoutHelper(QTextLayout *layout, QTextOption::WrapMode wrapM
   layout->setTextOption(option);
 
   QList<QTextLayout::FormatRange> formatRanges
-         = QtUi::style()->toTextLayoutList(data(MessageModel::FormatRole).value<UiStyle::FormatList>(), layout->text().length());
+         = QtUi::style()->toTextLayoutList(formatList(), layout->text().length(), data(ChatLineModel::MsgLabelRole).toUInt());
   layout->setAdditionalFormats(formatRanges);
 }
 
+UiStyle::FormatList ChatItem::formatList() const {
+  return data(MessageModel::FormatRole).value<UiStyle::FormatList>();
+}
+
 void ChatItem::doLayout(QTextLayout *layout) const {
   layout->beginLayout();
   QTextLine line = layout->createLine();
@@ -83,18 +102,27 @@ void ChatItem::doLayout(QTextLayout *layout) const {
   layout->endLayout();
 }
 
+void ChatItem::paintBackground(QPainter *painter) {
+  painter->setClipRect(boundingRect()); // no idea why QGraphicsItem clipping won't work
+
+  QVariant bgBrush;
+  if(_selectionMode == FullSelection)
+    bgBrush = data(ChatLineModel::SelectedBackgroundRole);
+  else
+    bgBrush = data(ChatLineModel::BackgroundRole);
+  if(bgBrush.isValid())
+    painter->fillRect(boundingRect(), bgBrush.value<QBrush>());
+}
 
 // NOTE: This is not the most time-efficient implementation, but it saves space by not caching unnecessary data
 //       This is a deliberate trade-off. (-> selectFmt creation, data() call)
 void ChatItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
   Q_UNUSED(option); Q_UNUSED(widget);
-  painter->setClipRect(boundingRect()); // no idea why QGraphicsItem clipping won't work
-  QVector<QTextLayout::FormatRange> formats = additionalFormats();
-  QTextLayout::FormatRange selectFmt = selectionFormat();
-  if(selectFmt.format.isValid()) formats.append(selectFmt);
+  paintBackground(painter);
+
   QTextLayout layout;
   initLayout(&layout);
-  layout.draw(painter, QPointF(0,0), formats, boundingRect());
+  layout.draw(painter, QPointF(0,0), additionalFormats(), boundingRect());
 
   //  layout()->draw(painter, QPointF(0,0), formats, boundingRect());
 
@@ -126,19 +154,58 @@ void ChatItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 //   painter->drawRect(_boundingRect.adjusted(0, 0, -1, -1));
 }
 
-qint16 ChatItem::posToCursor(const QPointF &pos) const {
-  if(pos.y() > height()) return data(MessageModel::DisplayRole).toString().length();
-  if(pos.y() < 0) return 0;
+void ChatItem::overlayFormat(UiStyle::FormatList &fmtList, int start, int end, quint32 overlayFmt) const {
+  for(int i = 0; i < fmtList.count(); i++) {
+    int fmtStart = fmtList.at(i).first;
+    int fmtEnd = (i < fmtList.count()-1 ? fmtList.at(i+1).first : data(MessageModel::DisplayRole).toString().length());
 
-  QTextLayout layout;
-  initLayout(&layout);
-  for(int l = layout.lineCount() - 1; l >= 0; l--) {
-    QTextLine line = layout.lineAt(l);
-    if(pos.y() >= line.y()) {
-      return line.xToCursor(pos.x(), QTextLine::CursorOnCharacter);
+    if(fmtEnd <= start)
+      continue;
+    if(fmtStart >= end)
+      break;
+
+    // split the format if necessary
+    if(fmtStart < start) {
+      fmtList.insert(i, fmtList.at(i));
+      fmtList[++i].first = start;
     }
+    if(end < fmtEnd) {
+      fmtList.insert(i, fmtList.at(i));
+      fmtList[i+1].first = end;
+    }
+
+    fmtList[i].second |= overlayFmt;
   }
-  return 0;
+}
+
+QVector<QTextLayout::FormatRange> ChatItem::additionalFormats() const {
+  return selectionFormats();
+}
+
+QVector<QTextLayout::FormatRange> ChatItem::selectionFormats() const {
+  if(!hasSelection())
+    return QVector<QTextLayout::FormatRange>();
+
+  int start, end;
+  if(_selectionMode == FullSelection) {
+    start = 0;
+    end = data(MessageModel::DisplayRole).toString().length();
+  } else {
+    start = qMin(_selectionStart, _selectionEnd);
+    end = qMax(_selectionStart, _selectionEnd);
+  }
+
+  UiStyle::FormatList fmtList = formatList();
+
+  while(fmtList.count() > 1 && fmtList.at(1).first <= start)
+    fmtList.removeFirst();
+
+  fmtList.first().first = start;
+
+  while(fmtList.count() > 1 && fmtList.last().first >= end)
+    fmtList.removeLast();
+
+  return QtUi::style()->toTextLayoutList(fmtList, end, UiStyle::Selected|data(ChatLineModel::MsgLabelRole).toUInt()).toVector();
 }
 
 bool ChatItem::hasSelection() const {
@@ -195,25 +262,6 @@ bool ChatItem::isPosOverSelection(const QPointF &pos) const {
   return false;
 }
 
-QTextLayout::FormatRange ChatItem::selectionFormat() const {
-  QTextLayout::FormatRange selectFmt;
-  if(_selectionMode != NoSelection) {
-    selectFmt.format.setForeground(QApplication::palette().brush(QPalette::HighlightedText));
-    selectFmt.format.setBackground(QApplication::palette().brush(QPalette::Highlight));
-    if(_selectionMode == PartialSelection) {
-      selectFmt.start = qMin(_selectionStart, _selectionEnd);
-      selectFmt.length = qAbs(_selectionStart - _selectionEnd);
-    } else { // FullSelection
-      selectFmt.start = 0;
-      selectFmt.length = data(MessageModel::DisplayRole).toString().length();
-    }
-  } else {
-    selectFmt.start = -1;
-    selectFmt.length = 0;
-  }
-  return selectFmt;
-}
-
 QList<QRectF> ChatItem::findWords(const QString &searchWord, Qt::CaseSensitivity caseSensitive) {
   QList<QRectF> resultList;
   const QAbstractItemModel *model_ = model();
@@ -298,8 +346,8 @@ void ChatItem::addActionsToMenu(QMenu *menu, const QPointF &pos) {
 
 void SenderChatItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
   Q_UNUSED(option); Q_UNUSED(widget);
+  paintBackground(painter);
 
-  painter->setClipRect(boundingRect()); // no idea why QGraphicsItem clipping won't work
   QTextLayout layout;
   initLayout(&layout);
   qreal layoutWidth = layout.minimumWidth();
@@ -309,15 +357,13 @@ void SenderChatItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
   else
     offset = qMax(layoutWidth - width(), (qreal)0);
 
-  QTextLayout::FormatRange selectFmt = selectionFormat();
-
   if(layoutWidth > width()) {
     // Draw a nice gradient for longer items
     // Qt's text drawing with a gradient brush sucks, so we use an alpha-channeled pixmap instead
     QPixmap pixmap(layout.boundingRect().toRect().size());
     pixmap.fill(Qt::transparent);
     QPainter pixPainter(&pixmap);
-    layout.draw(&pixPainter, QPointF(qMax(offset, (qreal)0), 0), QVector<QTextLayout::FormatRange>() << selectFmt);
+    layout.draw(&pixPainter, QPointF(qMax(offset, (qreal)0), 0), additionalFormats());
     pixPainter.end();
 
     // Create alpha channel mask
@@ -339,7 +385,7 @@ void SenderChatItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
     pixmap.setAlphaChannel(mask);
     painter->drawPixmap(0, 0, pixmap);
   } else {
-    layout.draw(painter, QPointF(0,0), QVector<QTextLayout::FormatRange>() << selectFmt, boundingRect());
+    layout.draw(painter, QPointF(0,0), additionalFormats(), boundingRect());
   }
 }
 
@@ -353,13 +399,13 @@ ContentsChatItem::ContentsChatItem(const qreal &width, const QPointF &pos, QGrap
   : ChatItem(0, 0, pos, parent),
     _data(0)
 {
-  const QAbstractItemModel *model_ = model();
-  QModelIndex index = model_->index(row(), column());
-  _fontMetrics = QtUi::style()->fontMetrics(model_->data(index, ChatLineModel::FormatRole).value<UiStyle::FormatList>().at(0).second);
-
   setGeometryByWidth(width);
 }
 
+QFontMetricsF *ContentsChatItem::fontMetrics() const {
+  return QtUi::style()->fontMetrics(data(ChatLineModel::FormatRole).value<UiStyle::FormatList>().at(0).second, 0);
+}
+
 ContentsChatItem::~ContentsChatItem() {
   delete _data;
 }
@@ -367,28 +413,32 @@ ContentsChatItem::~ContentsChatItem() {
 ContentsChatItemPrivate *ContentsChatItem::privateData() const {
   if(!_data) {
     ContentsChatItem *that = const_cast<ContentsChatItem *>(this);
-    that->_data = new ContentsChatItemPrivate(findClickables(), that);
+    that->_data = new ContentsChatItemPrivate(ClickableList::fromString(data(ChatLineModel::DisplayRole).toString()), that);
   }
   return _data;
 }
 
 qreal ContentsChatItem::setGeometryByWidth(qreal w) {
-  if(w != width()) {
+  // We use this for reloading layout info as well, so we can't bail out if the width doesn't change
+
+  // compute height
+  int lines = 1;
+  WrapColumnFinder finder(this);
+  while(finder.nextWrapColumn(w) > 0)
+    lines++;
+  qreal h = lines * fontMetrics()->lineSpacing();
+  delete _data;
+  _data = 0;
+
+  if(w != width() || h != height()) {
     prepareGeometryChange();
-    setWidth(w);
-    // compute height
-    int lines = 1;
-    WrapColumnFinder finder(this);
-    while(finder.nextWrapColumn() > 0)
-      lines++;
-    setHeight(lines * fontMetrics()->lineSpacing());
-    delete _data;
-    _data = 0;
+    setGeometry(w, h);
   }
-  return height();
+  return h;
 }
 
 void ContentsChatItem::doLayout(QTextLayout *layout) const {
+  // QString t = data(Qt::DisplayRole).toString(); bool d = t.contains("protien");
   ChatLineModel::WrapList wrapList = data(ChatLineModel::WrapListRole).value<ChatLineModel::WrapList>();
   if(!wrapList.count()) return; // empty chatitem
 
@@ -400,104 +450,53 @@ void ContentsChatItem::doLayout(QTextLayout *layout) const {
     if(!line.isValid())
       break;
 
-    int col = finder.nextWrapColumn();
-    line.setNumColumns(col >= 0 ? col - line.textStart() : layout->text().length());
+    int col = finder.nextWrapColumn(width());
+    if(col < 0)
+      col = layout->text().length();
+    int num = col - line.textStart();
+
+    line.setNumColumns(num);
+
+    // Sometimes, setNumColumns will create a line that's too long (cf. Qt bug 238249)
+    // We verify this and try setting the width again, making it shorter each time until the lengths match.
+    // Dead fugly, but seems to work…
+    for(int i = line.textLength()-1; i >= 0 && line.textLength() > num; i--) {
+      line.setNumColumns(i);
+    }
+    if(num != line.textLength()) {
+      qWarning() << "WARNING: Layout engine couldn't workaround Qt bug 238249, please report!";
+      // qDebug() << num << line.textLength() << t.mid(line.textStart(), line.textLength()) << t.mid(line.textStart() + line.textLength());
+    }
+
     line.setPosition(QPointF(0, h));
     h += fontMetrics()->lineSpacing();
   }
   layout->endLayout();
 }
 
-// NOTE: This method is not threadsafe and not reentrant!
-//       (RegExps are not constant while matching, and they are static here for efficiency)
-QList<ContentsChatItem::Clickable> ContentsChatItem::findClickables() const {
-  // For matching URLs
-  static QString urlEnd("(?:>|[,.;:\"]*\\s|\\b|$)");
-  static QString urlChars("(?:[,.;:]*[\\w\\-~@/?&=+$()!%#*|{}\\[\\]'])");
-
-  static QRegExp regExp[] = {
-    // URL
-    // QRegExp(QString("((?:https?://|s?ftp://|irc://|mailto:|www\\.)%1+|%1+\\.[a-z]{2,4}(?:?=/%1+|\\b))%2").arg(urlChars, urlEnd)),
-    QRegExp(QString("((?:(?:https?://|s?ftp://|irc://|gopher://|mailto:)|www)%1+)%2").arg(urlChars, urlEnd), Qt::CaseInsensitive),
-
-    // Channel name
-    // We don't match for channel names starting with + or &, because that gives us a lot of false positives.
-    QRegExp("((?:#|![A-Z0-9]{5})[^,:\\s]+(?::[^,:\\s]+)?)\\b", Qt::CaseInsensitive)
-
-    // TODO: Nicks, we'll need a filtering for only matching known nicknames further down if we do this
-  };
-
-  static const int regExpCount = 2;  // number of regexps in the array above
-
-  qint16 matches[] = { 0, 0, 0 };
-  qint16 matchEnd[] = { 0, 0, 0 };
-
-  QString str = data(ChatLineModel::DisplayRole).toString();
-
-  QList<Clickable> result;
-  qint16 idx = 0;
-  qint16 minidx;
-  int type = -1;
-
-  do {
-    type = -1;
-    minidx = str.length();
-    for(int i = 0; i < regExpCount; i++) {
-      if(matches[i] < 0 || matchEnd[i] > str.length()) continue;
-      if(idx >= matchEnd[i]) {
-        matches[i] = regExp[i].indexIn(str, qMax(matchEnd[i], idx));
-        if(matches[i] >= 0) matchEnd[i] = matches[i] + regExp[i].cap(1).length();
-      }
-      if(matches[i] >= 0 && matches[i] < minidx) {
-        minidx = matches[i];
-        type = i;
-      }
-    }
-    if(type >= 0) {
-      idx = matchEnd[type];
-      QString match = str.mid(matches[type], matchEnd[type] - matches[type]);
-      if(type == Clickable::Url && str.at(idx-1) == ')') {  // special case: closing paren only matches if we had an open one
-        if(!match.contains('(')) {
-          matchEnd[type]--;
-          match.chop(1);
-        }
-      }
-      if(type == Clickable::Channel) {
-        // don't make clickable if it could be a #number
-        if(QRegExp("^#\\d+$").exactMatch(match))
-          continue;
-      }
-      result.append(Clickable((Clickable::Type)type, matches[type], matchEnd[type] - matches[type]));
-    }
-  } while(type >= 0);
-
-  /* testing
-  if(!result.isEmpty()) qDebug() << str;
-  foreach(Clickable click, result) {
-    qDebug() << str.mid(click.start, click.length);
-  }
-  */
-  return result;
+Clickable ContentsChatItem::clickableAt(const QPointF &pos) const {
+  return privateData()->clickables.atCursorPos(posToCursor(pos));
 }
 
-ContentsChatItem::Clickable ContentsChatItem::clickableAt(const QPointF &pos) const {
-  qint16 idx = posToCursor(pos);
+UiStyle::FormatList ContentsChatItem::formatList() const {
+  UiStyle::FormatList fmtList = ChatItem::formatList();
   for(int i = 0; i < privateData()->clickables.count(); i++) {
     Clickable click = privateData()->clickables.at(i);
-    if(idx >= click.start && idx < click.start + click.length)
-      return click;
+    if(click.type() == Clickable::Url) {
+      overlayFormat(fmtList, click.start(), click.start() + click.length(), UiStyle::Url);
+    }
   }
-  return Clickable();
+  return fmtList;
 }
 
 QVector<QTextLayout::FormatRange> ContentsChatItem::additionalFormats() const {
+  QVector<QTextLayout::FormatRange> fmt = ChatItem::additionalFormats();
   // mark a clickable if hovered upon
-  QVector<QTextLayout::FormatRange> fmt;
   if(privateData()->currentClickable.isValid()) {
     Clickable click = privateData()->currentClickable;
     QTextLayout::FormatRange f;
-    f.start = click.start;
-    f.length = click.length;
+    f.start = click.start();
+    f.length = click.length();
     f.format.setFontUnderline(true);
     fmt.append(f);
   }
@@ -517,38 +516,20 @@ void ContentsChatItem::endHoverMode() {
 
 void ContentsChatItem::handleClick(const QPointF &pos, ChatScene::ClickMode clickMode) {
   if(clickMode == ChatScene::SingleClick) {
-    Clickable click = clickableAt(pos);
-    if(click.isValid()) {
-      QString str = data(ChatLineModel::DisplayRole).toString().mid(click.start, click.length);
-      switch(click.type) {
-        case Clickable::Url:
-          if(!str.contains("://"))
-            str = "http://" + str;
-          QDesktopServices::openUrl(QUrl::fromEncoded(str.toUtf8(), QUrl::TolerantMode));
-          break;
-        case Clickable::Channel: {
-          NetworkId networkId = Client::networkModel()->networkId(data(MessageModel::BufferIdRole).value<BufferId>());
-          BufferId bufId = Client::networkModel()->bufferId(networkId, str);
-          if(bufId.isValid()) {
-            QModelIndex targetIdx = Client::networkModel()->bufferIndex(bufId);
-            Client::bufferModel()->switchToBuffer(bufId);
-            if(!targetIdx.data(NetworkModel::ItemActiveRole).toBool())
-              Client::userInput(BufferInfo::fakeStatusBuffer(networkId), QString("/JOIN %1").arg(str));
-          } else
-              Client::userInput(BufferInfo::fakeStatusBuffer(networkId), QString("/JOIN %1").arg(str));
-          break;
-        }
-        default:
-          break;
-      }
+    qint16 idx = posToCursor(pos);
+    Clickable foo = privateData()->clickables.atCursorPos(idx);
+    if(foo.isValid()) {
+      NetworkId networkId = Client::networkModel()->networkId(data(MessageModel::BufferIdRole).value<BufferId>());
+      QString text = data(ChatLineModel::DisplayRole).toString();
+      foo.activate(networkId, text);
     }
   } else if(clickMode == ChatScene::DoubleClick) {
     chatScene()->setSelectingItem(this);
     setSelectionMode(PartialSelection);
     Clickable click = clickableAt(pos);
     if(click.isValid()) {
-      setSelectionStart(click.start);
-      setSelectionEnd(click.start + click.length);
+      setSelectionStart(click.start());
+      setSelectionEnd(click.start() + click.length());
     } else {
       // find word boundary
       QString str = data(ChatLineModel::DisplayRole).toString();
@@ -581,11 +562,11 @@ void ContentsChatItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event) {
   bool onClickable = false;
   Clickable click = clickableAt(event->pos());
   if(click.isValid()) {
-    if(click.type == Clickable::Url) {
+    if(click.type() == Clickable::Url) {
       onClickable = true;
       showWebPreview(click);
-    } else if(click.type == Clickable::Channel) {
-      QString name = data(ChatLineModel::DisplayRole).toString().mid(click.start, click.length);
+    } else if(click.type() == Clickable::Channel) {
+      QString name = data(ChatLineModel::DisplayRole).toString().mid(click.start(), click.length());
       // don't make clickable if it's our own name
       BufferId myId = data(MessageModel::BufferIdRole).value<BufferId>();
       if(Client::networkModel()->bufferName(myId) != name)
@@ -605,7 +586,7 @@ void ContentsChatItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event) {
 void ContentsChatItem::addActionsToMenu(QMenu *menu, const QPointF &pos) {
   if(privateData()->currentClickable.isValid()) {
     Clickable click = privateData()->currentClickable;
-    switch(click.type) {
+    switch(click.type()) {
       case Clickable::Url:
         privateData()->activeClickable = click;
         menu->addAction(SmallIcon("edit-copy"), tr("Copy Link Address"),
@@ -615,7 +596,7 @@ void ContentsChatItem::addActionsToMenu(QMenu *menu, const QPointF &pos) {
         // Hide existing menu actions, they confuse us when right-clicking on a clickable
         foreach(QAction *action, menu->actions())
           action->setVisible(false);
-        QString name = data(ChatLineModel::DisplayRole).toString().mid(click.start, click.length);
+        QString name = data(ChatLineModel::DisplayRole).toString().mid(click.start(), click.length());
         GraphicalUi::contextMenuActionProvider()->addActions(menu, chatScene()->filter(), data(MessageModel::BufferIdRole).value<BufferId>(), name);
         break;
       }
@@ -630,8 +611,8 @@ void ContentsChatItem::addActionsToMenu(QMenu *menu, const QPointF &pos) {
 
 void ContentsChatItem::copyLinkToClipboard() {
   Clickable click = privateData()->activeClickable;
-  if(click.isValid() && click.type == Clickable::Url) {
-    QString url = data(ChatLineModel::DisplayRole).toString().mid(click.start, click.length);
+  if(click.isValid() && click.type() == Clickable::Url) {
+    QString url = data(ChatLineModel::DisplayRole).toString().mid(click.start(), click.length());
     if(!url.contains("://"))
       url = "http://" + url;
     chatScene()->stringToClipboard(url);
@@ -646,18 +627,19 @@ void ContentsChatItem::showWebPreview(const Clickable &click) {
 #else
   QTextLayout layout;
   initLayout(&layout);
-  QTextLine line = layout.lineForTextPosition(click.start);
-  qreal x = line.cursorToX(click.start);
-  qreal width = line.cursorToX(click.start + click.length) - x;
+  QTextLine line = layout.lineForTextPosition(click.start());
+  qreal x = line.cursorToX(click.start());
+  qreal width = line.cursorToX(click.start() + click.length()) - x;
   qreal height = line.height();
   qreal y = height * line.lineNumber();
 
   QPointF topLeft = scenePos() + QPointF(x, y);
   QRectF urlRect = QRectF(topLeft.x(), topLeft.y(), width, height);
 
-  QString url = data(ChatLineModel::DisplayRole).toString().mid(click.start, click.length);
-  if(!url.contains("://"))
-    url = "http://" + url;
+  QString urlstr = data(ChatLineModel::DisplayRole).toString().mid(click.start(), click.length());
+  if(!urlstr.contains("://"))
+    urlstr= "http://" + urlstr;
+  QUrl url = QUrl::fromEncoded(urlstr.toUtf8(), QUrl::TolerantMode);
   chatScene()->loadWebPreview(this, url, urlRect);
 #endif
 }
@@ -682,12 +664,12 @@ ContentsChatItem::WrapColumnFinder::WrapColumnFinder(const ChatItem *_item)
 ContentsChatItem::WrapColumnFinder::~WrapColumnFinder() {
 }
 
-qint16 ContentsChatItem::WrapColumnFinder::nextWrapColumn() {
+qint16 ContentsChatItem::WrapColumnFinder::nextWrapColumn(qreal width) {
   if(wordidx >= wrapList.count())
     return -1;
 
   lineCount++;
-  qreal targetWidth = lineCount * item->width() + choppedTrailing;
+  qreal targetWidth = lineCount * width + choppedTrailing;
 
   qint16 start = wordidx;
   qint16 end = wrapList.count() - 1;
