@@ -108,6 +108,7 @@ public:
  public slots:
   void updateForViewport(qreal width, qreal height);
   void setWidth(qreal width);
+  void layout(int start, int end, qreal width);
 
   // these are used by the chatitems to notify the scene and manage selections
   void setSelectingItem(ChatItem *item);
@@ -121,7 +122,7 @@ public:
   void requestBacklog();
 
 #ifdef HAVE_WEBKIT
-  void loadWebPreview(ChatItem *parentItem, const QString &url, const QRectF &urlRect);
+  void loadWebPreview(ChatItem *parentItem, const QUrl &url, const QRectF &urlRect);
   void clearWebPreview(ChatItem *parentItem = 0);
 #endif
 
@@ -141,6 +142,7 @@ protected:
 protected slots:
   void rowsInserted(const QModelIndex &, int, int);
   void rowsAboutToBeRemoved(const QModelIndex &, int, int);
+  void dataChanged(const QModelIndex &, const QModelIndex &);
 
 private slots:
   void firstHandlePositionChanged(qreal xpos);
@@ -201,7 +203,7 @@ private:
     };
     ChatItem *parentItem;
     QGraphicsItem *previewItem;
-    QString url;
+    QUrl url;
     QRectF urlRect;
     PreviewState previewState;
     QTimer timer;
